package il.ac.tau.cs.sw1.wildworld;

/**
 * The <code>Action</code> interface represents an animal's action. When the
 * animal is requested to act its decision is encoded as an
 * <code>Action</code> instance. The animal's action consist of its movement
 * decision and of its spawning one. Each round an animal makes a decision as to
 * its action for this round, the result of this process is captured in an
 * object of this class.
 * 
 * @see Animal
 */
public interface Action {

	/**
	 * A set of constants representing the movement options for an animal.
	 */
	static final int STAY_PUT = 0;
	static final int NORTH = 0x1;
	static final int SOUTH = 0x2;
	static final int EAST = 0x4;
	static final int WEST = 0x8;
	static final int NORTH_EAST = NORTH | EAST;
	static final int NORTH_WEST = NORTH | WEST;
	static final int SOUTH_EAST = SOUTH | EAST;
	static final int SOUTH_WEST = SOUTH | WEST;

	/**
	 * Get the animal.
	 * 
	 * @post @ret != null
	 * 
	 * @return - the animal who created this action instance
	 */
	public Animal getAnimal();

	/**
	 * Get the animal's movement. An animal may movement is one of the defined
	 * movement constants.
	 * 
	 * @return one of the above defined constants
	 */
	public int getMovement();

	/**
	 * Set the animal's movement. An animal may movement is one of the defined
	 * movement constants.
	 */
	public void setMovement(int movement);

	/**
	 * Get the animal's decision whether or not to spawn. An animal can spawn
	 * only if it has more energy than the initial energy of the offspring. An
	 * animal must move once it has spawned.
	 * 
	 * @post $ret ==> getAnimal().getEnergy() > getAnimal.getSizeOfOffspring()
	 * @post $ret ==> getMovement() != Action.STAY_PUT
	 */
	public boolean getSpawn();

	/**
	 * Set the animal's decision whether or not to spawn. An animal can spawn
	 * only if it has more energy than the initial energy of the offspring. An
	 * animal must move once it has spawned.
	 * 
	 * @post $ret ==> getAnimal().getEnergy() > getAnimal.getSizeOfOffspring()
	 * @post $ret ==> getMovement() != Action.STAY_PUT
	 */
	public void setSpawn(boolean spawn);
}
