package il.ac.tau.cs.sw1.wildworld;

/**
 * Represents an animal in the system
 */
public interface Animal {

	/**
	 * Decide on the animal's next action. This method is called on each round
	 * and is where the animal should decide on its action for that round.
	 * 
	 * @post $ret != null
	 * @return - the outcome of the decision process.
	 * @see Action
	 */
	public Action act();

	/**
	 * Decrease the animal's energy level by a specified amount
	 * 
	 * @pre amount > 0
	 * @post getEnergy() == $prev(getEnergy()) - amount
	 * 
	 * @param amount
	 *            - the amount of energy to decrease.
	 */
	public void decreaseEnergy(double amount);

	/**
	 * Get the animal's energy level
	 * 
	 * @return - the animal's current energy level.
	 */
	public double getEnergy();

	/**
	 * Retrieve the name of the animal
	 * 
	 * @post $ret != null
	 * 
	 * @return The name of the animal (e.g "cow" , "lion")
	 */
	public String getName();

	/**
	 * Retrieve the range of sight of an animal.
	 * 
	 * @post 0 < $ret < getWorld().getSize()
	 * 
	 * @return An animal's range of sight.
	 */
	public int getRangeOfVision();

	/**
	 * Obtain the initial amount of energy an offspring of this animal type has.
	 * 
	 * @return an offspring's initial energy
	 */
	public double getSizeOfOffspring();

	/**
	 * Retrieves the world this animal inhabits
	 * 
	 * @post $ret != null
	 * 
	 * @return the world this animal inhabits
	 */
	public World getWorld();

	/**
	 * Increase the animal's energy level by a specified amount
	 * 
	 * @pre amount > 0
	 * @post getEnergy() == $prev(getEnergy()) + amount
	 * 
	 * @param amount
	 *            - the amount of energy to increase.
	 */
	public void increaseEnergy(double amount);

	/**
	 * Determine whether this animal is carnivore. An animal cannot be both
	 * herbivore and carnivore.
	 * 
	 * @see isHerbivore
	 */
	public boolean isCarnivore();

	/**
	 * Determine whether this animal is herbivore. An animal cannot be both
	 * herbivore and carnivore.
	 * 
	 * @see isCarnivore
	 */
	public boolean isHerbivore();

	/**
	 * @pre getEnergy() > getSizeOfOffspring()
	 * @post $ret != null
	 * @post $ret.getEnergy() == getSizeOfOffspring()
	 * @post getEnergy() == $prev(getEnergy() - getSizeOfOffspring())
	 * @post $ret.getName() == this.getName()
	 * 
	 * @return a new animal instance
	 */
	public Animal spawn();

}
